
#include "Arduino.h"
#define NECHANT 256
#define NECHANTD2 128
#define SHIFT_ACCUM 24

uint32_t icr;
uint32_t table_onde_A[NECHANT];
uint32_t table_onde_B[NECHANT];
uint32_t accum1,accum2,increm;
uint8_t indexA,indexB;
uint16_t diviseur[6] = {0,1,8,64,256,1024};
volatile uint8_t flash;
uint8_t strob = 0;
uint8_t synchro = 0;
float clock_period;
uint8_t front1 = 0;
uint8_t front2 = 0;
float duree_impulsion_4,duree_impulsion_5;

              
void init_pwm_timer1(uint32_t period) {
    char clockBits;
    TCCR1A = 0;
    TCCR1A |= (1 << COM1A1); //Clear OCnA/OCnB/OCnC on compare match, set OCnA/OCnB/OCnC at BOTTOM (non-inverting mode)
    TCCR1A |= (1 << COM1B1);
#if defined(__AVR_ATmega2560__) || defined(__AVR_ATmega32U4__)
    TCCR1A |= (1 << COM1C1);
#endif
    TCCR1B = 1 << WGM13; // phase and frequency correct pwm mode, top = ICR1
    int d = 1;
    icr = (F_CPU/1000000*period/2);
    while ((icr>0xFFFF)&&(d<6)) { // choix du diviseur d'horloge
        d++;
        icr = (F_CPU/1000000*period/2/diviseur[d]);
   } 
   clockBits = d;
   ICR1 = icr; // valeur maximale du compteur
   TIMSK1 = 1 << TOIE1; // overflow interrupt enable
   sei(); // activation des interruptions
   TCNT1 = 0; // mise à zéro du compteur
   TCCR1B |= clockBits; // déclenchement du compteur
}
                
ISR(TIMER1_OVF_vect) { // Timer 1 Overflow interrupt
  accum1 += increm;
  accum2 += increm;
  indexA = accum1 >> SHIFT_ACCUM;
  OCR1A = table_onde_A[indexA];
  indexB = accum2 >> SHIFT_ACCUM;
  OCR1B = table_onde_B[indexB];
  if (indexA==0) { // début période
 #if defined(__AVR_ATmega2560__)
      PORTH |= (1 << PORTH4); // flash ON
      PORTE |= (1 << PORTE4);
 #else
      PORTD |= (1 << PORTD7); // flash ON
      PORTD |= (1 << PORTD2);
      
 #endif
      flash = 1;
      }
  
  if ((flash)&&(indexA==2)) {
  #if defined(__AVR_ATmega2560__)
      if (strob) PORTH &= ~(1 << PORTH4); // flash OFF
  #else
      if (strob) PORTD &= ~(1 << PORTD7); // flash OFF
  #endif   
      flash = 0;
  }
  if (indexA==NECHANTD2) {
  #if defined(__AVR_ATmega2560__)
     PORTE &= ~(1 << PORTE4);
  #else
     PORTD &= ~(1 << PORTD2);
  #endif
  }
}
                  
void start_capture(uint8_t clock) {
    cli();
    TCCR5A = 0; // normal counting
    TCCR4A = 0;
    TCCR5B = (1 << ICES5); // détection d'un front montant
    TIMSK5 = (1 << ICIE5); // input capture interrupt enable interrupt
    TCCR4B = (1 << ICES4);
    TIMSK4 = (1 << ICIE4);
    sei();
    TCCR5B |= clock;
    TCCR4B |= clock;
    clock_period = 1.0/F_CPU*diviseur[clock];
}
              
ISR(TIMER5_CAPT_vect) {
   TCNT5 = 0;
   uint16_t count = ICR5; // input capture register
   float duree = count*clock_period*1000;
   if (front1==0) {
    TCCR5B &= ~(1 << ICES5);
    front1=1;
    Serial.println("Diode 1 : Phase = "+String(indexA)+" Duree = "+String(duree_impulsion_5)+" Intervalle = "+String(duree+duree_impulsion_5));
   }
   else {
      TCCR5B |= (1 << ICES5);
      front1=0;
      duree_impulsion_5 = duree;
    }
} 
               
ISR(TIMER4_CAPT_vect) {
   TCNT4 = 0;
   uint16_t count = ICR4; // input capture register
   float duree = count*clock_period*1000;
   if (front2==0) {
    TCCR4B &= ~(1 << ICES4);
    front2=1;
    Serial.println("Diode 2 : Phase = "+String(indexA)+" Duree = "+String(duree_impulsion_4)+" Intervalle = "+String(duree+duree_impulsion_4));
   }
   else {
      TCCR4B |= (1 << ICES4);
      front2=0;
      duree_impulsion_4 = duree;
      
    }
} 
               
void set_sinus_table_A(float amp, float offset) {
  int i;
  float dt = 2*3.1415926/NECHANT;
  for(i=0; i<NECHANT; i++) {
    table_onde_A[i] = icr*0.5*(1.0+offset+amp*sin(i*dt));
  }  
}

void set_sinus_table_B(float amp, float offset) {
  int i;
  float dt = 2*3.1415926/NECHANT;
  for(i=0; i<NECHANT; i++) {
    table_onde_B[i] = icr*0.5*(1.0+offset+amp*sin(i*dt));
  }  
}
                 
void setup() {
    Serial.begin(115200);
    pinMode(48,INPUT);
    pinMode(49,INPUT);
    pinMode(8,OUTPUT); 
    digitalWrite(8,HIGH); // commande ENABLE A
    pinMode(6,OUTPUT);
    digitalWrite(6,HIGH); // commande ENABLE B
    pinMode(3,OUTPUT); // alim sortie comparateur
    digitalWrite(3,HIGH);
#if defined(__AVR_ATmega2560__)
    pinMode(11,OUTPUT);
    pinMode(12,OUTPUT);
#elif defined(__AVR_ATmega32U4__)
    pinMode(9,OUTPUT);
    pinMode(10,OUTPUT);
#else
    pinMode(9,OUTPUT); 
    pinMode(10,OUTPUT);
#endif
    pinMode(7,OUTPUT);
    digitalWrite(7,HIGH); // PD7 sur UNO, PH4 sur MEGA
    pinMode(2,OUTPUT);
    digitalWrite(2,HIGH); // PD2 sur UNO, PE4 sur MEGA
    pinMode(4,OUTPUT);
    digitalWrite(4,HIGH);
    uint32_t period_pwm = 100; // en microsecondes 
    float frequence = 4; // en Hz
    accum1 = 0;
    accum2 = ((uint32_t)(NECHANT * 0.25)) << SHIFT_ACCUM;
    flash=0;
    synchro=0;
    increm = (uint32_t) (((float)(0xFFFFFFFF))*((float)(frequence)*1e-6*(float)(period_pwm))); // incrément de l'accumulateur de phase
    init_pwm_timer1(period_pwm);
    set_sinus_table_A(0.5,0.5);
    set_sinus_table_B(0.5,0.0);
    front1 = front2 = 0;
    start_capture(4);
}
                  
void loop() {

}
                  